{% extends "timepiece/business/base.html" %}
{% load timepiece_tags %}
{% load url from future %}

{% block title %}{{ object.name }}{% endblock title %}

{% block crumbs %}
    {{ block.super }}
    <li>
        <span class="divider">/</span>
        <a href="">{{ object.name }}</a>
    </li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>{{ object.name }}</h2>
            <ul class="nav nav-pills">
                {% if perms.crm.change_business %}
                    <li><a href='{% url 'edit_business' object.pk %}'>Edit</a></li>
                    <li><a href="{% url 'delete_business' object.pk %}">Delete</a></li>
                {% endif %}
            </ul>
        </div>
    </div>

    <div class="row-fluid">
        <div class="span6">
            <table class="table table-bordered">
                <tr><th>Short name:</th><td>{{ object.short_name }}</td></tr>
                <tr><th>Email:</th><td>{{ object.email }}</td></tr>
                <tr><th>Description:</th><td>{{ object.description|linebreaks }}</td></tr>
                <tr><th>Notes:</th><td>{{ object.notes|linebreaks }}</td></tr>
            </table>
        </div>
    </div>
{% endblock content %}
