{% extends "timepiece/base.html" %}
{% load url from future %}

{% block crumbs %}
    {{ block.super }}
    <li>
        <span class="divider">/</span>
        <a href="{% url 'list_businesses' %}">Businesses</a>
    </li>
{% endblock crumbs %}
