{% extends "timepiece/project/base.html" %}
{% load url from future %}

{% block title %}Remove Project Relationship{% endblock title %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="">Remove Relationship</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Remove Project Relationship</h2>
            <br style="clear: both;" />
            <form class="form-inline" id="remove-project-relationship" action="" method="post" accept-charset="utf-8">
                {% csrf_token %}
                <h3>Are you sure you want to remove the assignment of {{ object.project }} to {{ object.user.get_name_or_username }}?</h3>
                <div class="form-actions">
                    <input class="btn btn-primary" type='submit' name='submit' value='Confirm' />
                    <input class="btn" type='button' value='Cancel' onclick='history.go(-1)' />
                </div>
            </form>
        </div>
    </div>
{% endblock content %}
