<table class="table table-bordered">
    <tr>
        <th>Activity Type</th>
        <th>Total Hours</th>
    </tr>
    {% for name, hours_activities in totals %}
        <tr class="ledger_header">
            <th>{{ name|title }}</th>
            <td class="hours"> {{ hours_activities.0 }} </td>
        </tr>
        {% for name, hours in hours_activities.1 %}
            <tr>
                <td>{{ name|title }}</td>
                <td class="hours">{{ hours }} </td>
            </tr>
        {% endfor %}
    {% endfor %}
</table>
