{% if billable_entries %}
    {% with entries=billable_entries %}
        <h4>Billable entries</h4>
        {% include 'timepiece/invoice/_weekly_entry_list_table.html' %}
    {% endwith %}
{% else %}
    <p>No billable entries were found.</p>
{% endif %}
{% if nonbillable_entries %}
    {% with entries=nonbillable_entries %}
        <h4>Non-billable entries</h4>
        {% include 'timepiece/invoice/_weekly_entry_list_table.html' %}
    {% endwith %}
{% else %}
    <p>No non-billable entries were found.</p>
{% endif %}
