{% comment %}
You can override this template to include extra, custom items in the timepiece
navigation bar.

We use Bootstrap styling so you can use both plain navigation items and
dropdown navigation items.

If you add many items, you may want to adjust the width at which the navbar
collapses by setting the variable @navbarCollapseWidth in your less file.

For example:

<!-- A dropdown menu item. -->
<li class="dropdown">
    <a href="#" class="dropdown-toggle" data-toggle="dropdown">Paste Bin <b class="caret"></b></a>
    <ul class="dropdown-menu">
        <li><a href="{% url 'create_paste' %}">Create Paste</a></li>
        <li><a href="{% url 'list_pastes' %}">View Pastes</a></li>
    </ul>
</li>

<!-- A plain menu item. -->
<li>
    <a href="http://example.com/">Example</a>
</li>

{% endcomment %}
